@@ -40,6 +40,7 @@
 
 #include "mozplugger.h"
 #include "child.h"
+#include "debug.h"
 
 #define MAX_CONTROLS_WIDTH           300
 #define MAX_CONTROLS_HEIGHT          100
@@ -668,8 +669,6 @@ static void exitEarly(void)
 
 /******************************************************************************/
 /**
- * @brief main
- *
  * mozplugger-controller takes two arguments. The first is a shell command line
  * to run when the 'play' button is pressed (or if autostart is set). The second
  * parameter is optional and is the ID of the parent window into which to draw
@@ -678,6 +677,74 @@ static void exitEarly(void)
  * If the command line contains the string "$window", then no controls are 
  * drawn.
  *
+ * @param[in]  argc    The number of arguments
+ * @param[in]  argv    Array of the arguments
+ * @param[out] pWidth  Width of window
+ * @param[out] pHeight Height of window
+ * @param[out] pWindow The Window ID
+ * @param[out] command The command line
+ *
+ * @return True if all Ok
+ *
+ ******************************************************************************/
+static int readCommandLine(int argc, char **argv, unsigned int * pWidth,
+                                                   unsigned int * pHeight,
+                                                   Window * pWindow,
+                                                   char ** pCommand)
+{
+     unsigned long temp = 0;
+     int x, y ;
+     int i;
+     char * fileName;
+
+     D("Controller started.....\n");
+
+     if (argc < 3)
+     {
+          return 0;
+     }
+
+     /* Global variables set are:
+        - repeatsResetVal
+        - pipe_fd
+        - flags */
+
+     i = sscanf(argv[1],"%d,%d,%d,%lu,%d,%d,%d,%d",
+            &flags,
+            &repeatsResetVal,
+            &pipe_fd,
+            &temp,
+            (int *)&x,  /* Not needed and not used */
+            (int *)&y,  /* Not needed and not used */
+            (int *)pWidth,
+            (int *)pHeight);
+
+     if(i < 8)
+     {
+          return 0;
+     }
+
+     *pWindow = (Window)temp;
+     *pCommand = argv[2];
+
+     fileName = getenv("file"); 
+
+     D("CONTROLLER: %s %s %s %s\n",
+       argv[0],
+       argv[1],
+       fileName ? fileName : "NULL",
+       argv[2]);
+
+     return 1;
+}
+
+/******************************************************************************/
+/**
+ * mozplugger-controller main()
+ *
+ * If the command line contains the string "$window", then no controls are 
+ * drawn.
+ *
  * @param[in] argc The number of arguments
  * @param[in] argv Array of the arguments
  *
@@ -691,49 +758,25 @@ int main(int argc, char **argv)
      /* Set defaults, may be changed later */
      unsigned int width =  DEFAULT_CONTROLS_WIDTH; 
      unsigned int height = DEFAULT_CONTROLS_HEIGHT;
+     int x, y;
 
-     Window parentWid;
-     unsigned long temp = 0;
-     int x, y ;
-     char * command;
-     int i;
+     Window parentWid = 0;
+     char * command = 0;
 
      XColor colour;
      XClassHint classhint;
      XSetWindowAttributes attr;
      XSizeHints wmHints;
 
-     D("Controller started.....\n");
-
-     if (argc < 3)
+     if(!readCommandLine(argc, argv, &width, 
+                                 &height, 
+                                 &parentWid,
+                                 &command))
      {
-          exitEarly();
+         exitEarly();
      }
 
-     i = sscanf(argv[1],"%d,%d,%d,%lu,%d,%d,%d,%d",
-            &flags,
-            &repeatsResetVal,
-            &pipe_fd,
-            &temp,
-            (int *)&x,  /* Not needed and not used */
-            (int *)&y,  /* Not needed and not used */
-            (int *)&width,
-            (int *)&height);
-
-     if(i < 8)
-     {
-          exitEarly();
-     }
-
-     parentWid = (Window)temp;
-     command = argv[2];
-
-     D("CONTROLLER: %s %s %s %s\n",
-       argv[0],
-       argv[1],
-       getenv("file"),
-       command);
-
+    
      /* The application will use the $repeat variable */
      if (flags & H_REPEATCOUNT)
      {   
