@@ -44,6 +44,7 @@
 
 #include "mozplugger.h"
 #include "child.h"
+#include "debug.h"
 
 /*****************************************************************************
  * Type declarations
@@ -114,11 +115,9 @@ static Window possible_victim_windows[MAX_POSS_VICTIMS];
  *****************************************************************************/
 static int error_handler(Display *dpy, XErrorEvent *err)
 {
-#ifdef DEBUG
      char buffer[1024];
      XGetErrorText(dpy, err->error_code, buffer, sizeof(buffer));
      D("!!!ERROR_HANDLER!!!: %s\n", buffer);
-#endif
      return 0;
 }
 
@@ -629,7 +628,6 @@ static void change_leader(void)
      XWMHints *leader_change;
      if ((leader_change = XGetWMHints(display, victimDetails.window)))
      {
-#ifdef DEBUG
           if((leader_change->flags & WindowGroupHint) != 0)
           {
 		D("Old window leader was 0x%x\n", 
@@ -644,7 +642,7 @@ static void change_leader(void)
 		D("InitialState hint = %i\n", 
                                        (unsigned) leader_change->initial_state);
           }
-#endif
+
 	  leader_change->flags |= WindowGroupHint;
 	  leader_change->window_group = wattr.root;
 
@@ -1041,7 +1039,6 @@ static void handle_rootWindow_event(const XEvent * const ev)
 	  break;
 
 
-#ifdef DEBUG
      case ConfigureNotify:
           D("***ConfigureNotify for root, window=0x%x, "
                       "x=%i, y=%i, width=%i, height=%i, override_redirect=%i\n",
@@ -1070,7 +1067,6 @@ static void handle_rootWindow_event(const XEvent * const ev)
      case ClientMessage:
           D("***ClientMessage for root\n");
           break;
-#endif
 
      default: 
           D("!!Got unhandled event for root->%d\n", ev->type);
@@ -1146,7 +1142,6 @@ static void handle_victimWindow_event(const XEvent * const ev)
 	  }
 	  break;
 
-#ifdef DEBUG
      case ConfigureNotify:
           D("CONFIGURE NOTIFY for victim, window=0x%x," 
                                     " x=%d, y=%d, w=%d, h=%d, "
@@ -1163,7 +1158,7 @@ static void handle_victimWindow_event(const XEvent * const ev)
           D("CLIENT MESSAGE for victim\n");
                     /* I see this with evince! perhaps it needs to be handled? */
           break;
-#endif
+
      case DestroyNotify:
           D("DESTROY NOTIFY for victim, window=0x%x\n", 
                                           (unsigned) ev->xdestroywindow.window);
