@@ -122,6 +122,12 @@ typedef struct
      short exists;
 } cacheEntry_t;
 
+typedef struct
+{
+    struct NPObject objHead;
+    NPP             assocInstance;
+} our_NPObject_t;
+
 /*****************************************************************************
  * Global variables
  *****************************************************************************/
@@ -1357,6 +1363,57 @@ static int safeURL(const char* url)
      return 1;
 }
 
+/*****************************************************************************/
+/**
+ * Extract the file name from the HTTP Headers (if present). This is passed
+ * via the fragment environment variable to the helper application. Copy
+ * a suitable filename for the URL to be used for the cached temporay file
+ * name.
+ *
+ * @param[in] THIS        Pointer to the instance data
+ * @param[in] headers         The HTTP headers to parse
+ * @param[out] fileName   The file Name extracted from the headers
+ *
+ * @return None
+ *
+ *****************************************************************************/
+static void parseHeaders(data_t * const THIS, const char * headers, char * fileName,
+                                                                 int maxFileNameLen)
+{
+     const char * p = headers;
+
+     if (!headers)
+     {
+          return;
+     }
+
+     while((p = strstr(p, "Content-Disposition:")) != NULL)
+     {
+          size_t len = strcspn(p, "\n\r");
+	  const char * start = strstr(p, "filename=\"");
+
+	  if (len == 0)
+          {
+	       break;
+          }
+
+          if((start ==0) || (start - p > len))
+          {
+	       p += len;
+	       continue;
+	  }
+ 	  start += strlen("filename=\"");
+	  len = len - (start - p) - 1;
+
+	  if(len > 0) 
+          {
+		strncpy(fileName, start, len);
+		fileName[len] = '\0';
+	  }
+	  p += len;
+     }
+}
+
 /*****************************************************************************/
 /**
  * Extract the 'fragment' from the end of the URL if present. This is passed
@@ -1726,6 +1783,27 @@ char *NPP_GetMIMEDescription(void)
      return x;
 }
 
+/*****************************************************************************/
+/**
+ * Provides plugin HasMethod that can be called from broswer
+ *
+ * @return True if has that method
+ *
+ *****************************************************************************/
+static void debugLogIdentifier(NPIdentifier name)
+{
+     if(NPN_IdentifierIsString(name))
+     {
+         char * str = NPN_UTF8FromIdentifier(name);
+         D("NPIdentifier = %s\n", str ? (char *)str : "NULL");
+         NPN_MemFree(str);
+     }
+     else
+     {
+         D("NPIdentifier = %i\n", NPN_IntFromIdentifier(name));
+     }
+}
+
 /*****************************************************************************/
 /**
  * Provides plugin HasMethod that can be called from broswer
@@ -1735,7 +1813,9 @@ char *NPP_GetMIMEDescription(void)
  *****************************************************************************/
 bool NPP_HasMethod(NPObject *npobj, NPIdentifier name)
 {
-     D("NPP_HasMethod called - %s\n", name ? (char *)name : "NULL");
+     D("NPP_HasMethod called\n");
+     debugLogIdentifier(name);
+
      return 0;
 }
 
@@ -1749,7 +1829,9 @@ bool NPP_HasMethod(NPObject *npobj, NPIdentifier name)
 bool NPP_Invoke(NPObject *npobj, NPIdentifier name,
                    const NPVariant *args, uint32_t argCount, NPVariant *result)
 {
-     D("NPP_Invoke called - %s\n", name ? (char *)name : "NULL");
+     D("NPP_Invoke called\n");
+     debugLogIdentifier(name);
+
      return 0;
 }
 
@@ -1762,8 +1844,21 @@ bool NPP_Invoke(NPObject *npobj, NPIdentifier name,
  *****************************************************************************/
 bool NPP_HasProperty(NPObject *npobj, NPIdentifier name)
 {
-     D("NPP_HasProperty called - %s\n", name ? (char *)name : "NULL");
-     return 0;
+     bool retVal = 0;
+
+     D("NPP_HasProperty called\n");
+     debugLogIdentifier(name);
+
+     char * str = NPN_UTF8FromIdentifier(name);
+     if(str)
+     {
+          if (strcasecmp("isplaying", str) == 0)
+	  {
+               retVal = 1;
+	  }
+          NPN_MemFree(str);
+     } 
+     return retVal;
 }
 
 /*****************************************************************************/
@@ -1775,8 +1870,42 @@ bool NPP_HasProperty(NPObject *npobj, NPIdentifier name)
  *****************************************************************************/
 bool NPP_GetProperty(NPObject *npobj, NPIdentifier name, NPVariant *result)
 {
-     D("NPP_GetProperty called - %s\n", name ? (char *)name : "NULL");
-     return 0;
+     bool retVal = 0;
+
+     D("NPP_GetProperty called\n");
+     debugLogIdentifier(name);
+
+     char * str = NPN_UTF8FromIdentifier(name);
+     if(str)
+     {
+          if (strcasecmp("isplaying", str) == 0)
+          {
+	       result->type  = NPVariantType_Bool;
+               result->value.boolValue = 0;
+               retVal = 1;
+ 	  
+               NPP instance = ((our_NPObject_t *)npobj)->assocInstance;
+               if(instance)
+               {
+                    data_t * THIS = instance->pdata;
+                    if (THIS)
+                    {
+                         if((THIS->commsPipeFd >= 0) || (THIS->pid > -1))
+                         {             
+                             int status;
+                             if(waitpid(THIS->pid, &status, WNOHANG) == 0)
+                             {
+                                 /* If no status available from child then child
+                                  * must still be running!? */
+                                 result->value.boolValue = 1;
+                             }
+                         }
+                    }
+               }
+          }
+          NPN_MemFree(str);
+     } 
+     return retVal;
 }
 
 /*****************************************************************************/
@@ -1789,7 +1918,9 @@ bool NPP_GetProperty(NPObject *npobj, NPIdentifier name, NPVariant *result)
 bool NPP_SetProperty(NPObject *npobj, NPIdentifier name,
                                                          const NPVariant *value)
 {
-     D("NPP_SetProperty called - %s\n", name ? (char *)name : "NULL");
+     D("NPP_SetProperty called\n");
+     debugLogIdentifier(name);
+
      return 0;
 }
 
@@ -1883,6 +2014,20 @@ static NPBool getPluginNeedsXembed(NPP instance, NPError *pErr)
      }
 }
 
+/*****************************************************************************/
+/**
+ * Create a JavaScript object
+ *
+ * @return Pointer to created object
+ *
+ *****************************************************************************/
+static NPObject * NPP_AllocateObj(NPP instance, NPClass * aClass)
+{
+    our_NPObject_t * pObj = NPN_MemAlloc(sizeof(our_NPObject_t));
+    pObj->assocInstance = instance;
+    return (NPObject *) pObj;
+}
+
 /*****************************************************************************/
 /**
  * Get plugin Scritable Object
@@ -1903,6 +2048,7 @@ static NPObject * getPluginScritableObject(NPP instance, NPError * pErr)
 
      pluginClass.structVersion = NP_CLASS_STRUCT_VERSION;
 
+     pluginClass.allocate = NPP_AllocateObj;
      pluginClass.hasMethod = NPP_HasMethod;
      pluginClass.invoke = NPP_Invoke;
      pluginClass.hasProperty = NPP_HasProperty;
@@ -2328,14 +2474,17 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
  	  if (THIS->pid > 0)
           {
 	       my_kill(-THIS->pid);
+               THIS->pid = 0;
           }
 	  if (THIS->commsPipeFd >= 0)
           {
 	       close(THIS->commsPipeFd);
+               THIS->commsPipeFd = -1;
           }
           if(THIS->tmpFileFd >= 0)
           {
                close(THIS->tmpFileFd);
+               THIS->tmpFileFd = -1;
           }
           if(THIS->tmpFileName != 0)
           {
@@ -2490,6 +2639,51 @@ static void new_child(NPP instance, const char* fname)
      close(commsPipe[1]);
 }
 
+/*****************************************************************************/
+/**
+ * Guess a temporary file name
+ *
+ * @return file descriptor
+ *
+ *****************************************************************************/
+static int guessTmpFile(const char * fileName, int soFar,
+                                     char * tmpFilePath, int maxTmpFilePathLen)
+{
+     int i;
+     int fd = -1;
+
+     for(i = 0; i <= 100; i++)
+     {
+          if(i == 0)
+	  {
+	       strncpy(&tmpFilePath[soFar], fileName,
+				maxTmpFilePathLen - soFar);
+          }
+	  else if (i == 100)
+	  {
+               strncpy(&tmpFilePath[soFar], "XXXXXX",
+                                                     maxTmpFilePathLen - soFar);
+               fd = mkstemp(tmpFilePath);
+	       break;
+	  }
+	  else
+	  {
+	       snprintf(&tmpFilePath[soFar],
+				 maxTmpFilePathLen - soFar,
+				 "%03i-%s", i, fileName);
+	  }
+
+	  fd = open(tmpFilePath, O_CREAT | O_EXCL | O_WRONLY,
+			  S_IRUSR | S_IWUSR);
+	  if(fd >= 0)
+	  {
+	       break;
+	  }
+     }
+
+     return fd;
+}
+
 /*****************************************************************************/
 /**
  * From the url create a temporary file to hold a copy of th URL contents.
@@ -2510,80 +2704,45 @@ static int createTmpFile(const char * fileName,
 
      D("Creating temp file for '%s'\n", fileName);
 
-     root = getenv("MOZPLUGGER_HOME");
-     if(root == 0)
-     {
-         root = getenv("HOME");
-         if(root)
-         {
-              snprintf(tmpFilePath, maxTmpFilePathLen, "%s/.mozplugger", root);
-              if((mkdir(tmpFilePath, S_IRWXU ) != 0) && (errno != EEXIST))
-              {
-                  root = NULL;
-              }
-         }
-     }
-     else
-     {
-         strncpy(tmpFilePath, root, maxTmpFilePathLen);
-     }
-
+     root = getenv("MOZPLUGGER_TMP");
      if(root)
      {
-          int soFar = strlen(tmpFilePath);
+          int soFar;
+
+          strncpy(tmpFilePath, root, maxTmpFilePathLen);
+          soFar = strlen(tmpFilePath);
+
 	  soFar += snprintf(&tmpFilePath[soFar], maxTmpFilePathLen-soFar, 
                                                                 "/tmp-%i", pid);
           if( (mkdir(tmpFilePath, S_IRWXU) == 0) || (errno == EEXIST))
           {
+               D("Creating temp file in '%s'\n", tmpFilePath);
+
                tmpFilePath[soFar++] = '/';
 
-               strncpy(&tmpFilePath[soFar], fileName, maxTmpFilePathLen - soFar); 
-               fd = open(tmpFilePath, O_CREAT | O_EXCL | O_WRONLY, 
-                                                             S_IRUSR | S_IWUSR);
-               if(fd < 0)
-               {
-                     strncpy(&tmpFilePath[soFar], "XXXXXX", 
-                                                     maxTmpFilePathLen - soFar);
-                     fd = mkstemp(tmpFilePath);
-               }
+	       fd = guessTmpFile(fileName, soFar, tmpFilePath, maxTmpFilePathLen);
           }
      }
 
      if(fd < 0)
      {
           root = getenv("TMPDIR");
-          if(root)
+          if(!root)
           {
-               snprintf(tmpFilePath, maxTmpFilePathLen, "%s/mozplugger-%i", 
+               root = "/tmp";
+          }
+
+          snprintf(tmpFilePath, maxTmpFilePathLen, "%s/mozplugger-%i", 
                                                                      root, pid);
-               if((mkdir(tmpFilePath, S_IRWXU) == 0) || (errno == EEXIST))
-               {
-                    int i;
-                    int soFar = strlen(tmpFilePath);
-                    tmpFilePath[soFar++] = '/';
+          if((mkdir(tmpFilePath, S_IRWXU) == 0) || (errno == EEXIST))
+          {
+                int soFar = strlen(tmpFilePath);
 
-                    for(i = 0; i < 100; i++)
-                    {
-                         if(i == 0) 
-                         {
-                              strncpy(&tmpFilePath[soFar], fileName, 
-                                                     maxTmpFilePathLen - soFar); 
-                         }
-                         else
-                         {
-                               snprintf(&tmpFilePath[soFar], 
-                                                      maxTmpFilePathLen - soFar,
-                                                        "%03i-%s", i, fileName);
-                         }
+                D("Creating temp file in '%s'\n", tmpFilePath);
 
-                         fd = open(tmpFilePath, O_CREAT | O_EXCL | O_WRONLY, 
-                                                             S_IRUSR | S_IWUSR);
-                         if(fd >= 0)
-                         {
-                             break;
-                         }
-                    }
-               }
+                tmpFilePath[soFar++] = '/';
+
+	        fd = guessTmpFile(fileName, soFar, tmpFilePath, maxTmpFilePathLen);
           }
      }
 
@@ -2683,6 +2842,9 @@ NPError NPP_NewStream(NPP instance,
      /* Extract from the URL the various additional information */
      parseURL(THIS, stream->url, fileName, sizeof(fileName)/sizeof(char)-1);
 
+     /* Extract the fileName from HTTP headers, overide URL fileName */
+     parseHeaders(THIS, stream->headers, fileName, sizeof(fileName)/sizeof(char)-1);
+
      if(THIS->command == 0)
      {
 	  NPN_Status(instance, 
@@ -2856,9 +3018,16 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
 
      if (THIS->commsPipeFd >= 0)
      {
+          ssize_t ret;
 	  D("Writing WIN 0x%x to fd %d\n", (unsigned) window->window, 
                                                              THIS->commsPipeFd);
-	  write(THIS->commsPipeFd, (char *)window, sizeof(*window));
+	  ret = write(THIS->commsPipeFd, (char *)window, sizeof(*window));
+          if(ret < sizeof(*window))
+          {
+               D("Writing to comms pipe failed\n");
+               close(THIS->commsPipeFd);
+               THIS->commsPipeFd = -1;
+          }
      }
 
      resize_window(instance);
