@@ -35,6 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#ifndef npapi_h_
+#define npapi_h_
+
+#include "nptypes.h"
 
 /*
  *  npapi.h Mozilla 3.48 But with MACOS and WINDOWS stuff removed and
@@ -42,13 +46,10 @@
  *  Netscape client plug-in API spec
  */
 
-#ifndef _NPAPI_H_
-#define _NPAPI_H_
-
 #if defined(XP_UNIX) 
-#	include <stdio.h>
-#	include <X11/Xlib.h>
-#	include <X11/Xutil.h>
+#  include <stdio.h>
+#  include <X11/Xlib.h>
+#  include <X11/Xutil.h>
 #endif
 
 
@@ -57,10 +58,7 @@
 /*----------------------------------------------------------------------*/
 
 #define NP_VERSION_MAJOR 0
-#define NP_VERSION_MINOR 11
-// #define NP_VERSION_MINOR 22     // Latest is at version 22!
-
-#include "nptypes.h"
+#define NP_VERSION_MINOR 23
 
 /*----------------------------------------------------------------------*/
 /*                   Definition of Basic Types                          */
@@ -182,7 +180,7 @@ typedef struct
  *   compatible with older compilers. To prevent older plugins from 
  *   not understanding a new browser's ABI, these masks change the 
  *   values of those selectors on those platforms. To remain backwards
- *   compatible with differenet versions of the browser, plugins can 
+ *   compatible with different versions of the browser, plugins can 
  *   use these masks to dynamically determine and use the correct C++
  *   ABI that the browser is expecting. This does not apply to Windows 
  *   as Microsoft's COM ABI will likely not change.
@@ -199,13 +197,7 @@ typedef struct
 #define _NP_ABI_MIXIN_FOR_GCC3 0
 #endif
 
-#ifdef XP_MACOSX
-#define NP_ABI_MACHO_MASK 0x01000000
-#define _NP_ABI_MIXIN_FOR_MACHO NP_ABI_MACHO_MASK
-#else
 #define _NP_ABI_MIXIN_FOR_MACHO 0
-#endif
-
 
 #define NP_ABI_MASK (_NP_ABI_MIXIN_FOR_GCC3 | _NP_ABI_MIXIN_FOR_MACHO)
 
@@ -250,6 +242,9 @@ typedef enum {
    */
   NPPVpluginWantsAllNetworkStreams = 18
 
+#ifdef MOZ_PLATFORM_HILDON
+  , NPPVpluginWindowlessLocalBool = 2002
+#endif
 } NPPVariable;
 
 /*
@@ -279,6 +274,10 @@ typedef enum {
   NPNVSupportsWindowless = 17,
 
   NPNVprivateModeBool = 18
+
+#ifdef MOZ_PLATFORM_HILDON
+  , NPNVSupportsWindowlessLocal = 2002
+#endif
 } NPNVariable;
 
 typedef enum {
@@ -286,7 +285,7 @@ typedef enum {
   NPNURLVProxy
 } NPNURLVariable;
 /*
- * The type of Tookkit the widgets use
+ * The type of Toolkit the widgets use
  */
 typedef enum {
   NPNVGtk12 = 1,
@@ -304,19 +303,35 @@ typedef enum {
 
 typedef struct _NPWindow 
 {
-    void*	window;		/* Platform specific window handle */
-    int32_t	x;		/* Position of top left corner relative */
-    int32_t	y;		/* to a netscape page. */
-    uint32_t	width;		/* Maximum window size */
-    uint32_t	height;
-    NPRect	clipRect;	/* Clipping rectangle in port coordinates */
-				/* Used by MAC only. */
+  void* window;  /* Platform specific window handle */
+                 /* OS/2: x - Position of bottom left corner */
+                 /* OS/2: y - relative to visible netscape window */
+  int32_t  x;      /* Position of top left corner relative */
+  int32_t  y;      /* to a netscape page. */
+  uint32_t width;  /* Maximum window size */
+  uint32_t height;
+  NPRect   clipRect; /* Clipping rectangle in port coordinates */
 #if defined(XP_UNIX)
-    void *	ws_info;	/* Platform-dependent additonal data */
+  void * ws_info; /* Platform-dependent additional data */
 #endif /* XP_UNIX */
   NPWindowType type; /* Is this a window or a drawable? */
 } NPWindow;
 
+typedef struct _NPImageExpose
+{
+  char*    data;       /* image pointer */
+  int32_t  stride;     /* Stride of data image pointer */
+  int32_t  depth;      /* Depth of image pointer */
+  int32_t  x;          /* Expose x */
+  int32_t  y;          /* Expose y */
+  uint32_t width;      /* Expose width */
+  uint32_t height;     /* Expose height */
+  NPSize   dataSize;   /* Data buffer size */
+  float    translateX; /* translate X matrix value */
+  float    translateY; /* translate Y matrix value */
+  float    scaleX;     /* scale X matrix value */
+  float    scaleY;     /* scale Y matrix value */
+} NPImageExpose;
 
 typedef struct _NPFullPrint
 {
@@ -354,7 +369,19 @@ typedef Region NPRegion;
 typedef void *NPRegion;
 #endif
 
+typedef struct _NPNSString NPNSString;
+typedef struct _NPNSWindow NPNSWindow;
+typedef struct _NPNSMenu   NPNSMenu;
 
+typedef void *NPMenu;
+
+typedef enum {
+  NPCoordinateSpacePlugin = 1,
+  NPCoordinateSpaceWindow,
+  NPCoordinateSpaceFlippedWindow,
+  NPCoordinateSpaceScreen,
+  NPCoordinateSpaceFlippedScreen
+} NPCoordinateSpace;
 
 /*
  * Values for mode passed to NPP_New:
@@ -433,11 +460,7 @@ typedef void *NPRegion;
 /*                        Function Prototypes                           */
 /*----------------------------------------------------------------------*/
 
-#if defined(__OS2__)
-#define NP_LOADDS  _System
-#else
 #define NP_LOADDS
-#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -446,11 +469,11 @@ extern "C" {
 /* NPP_* functions are provided by the plugin and called by the navigator. */
 
 #ifdef XP_UNIX
-char* NPP_GetMIMEDescription(void);
+char* NPP_GetMIMEDescription();
 #endif
 
-NPError NP_LOADDS NPP_Initialize(void);
-void    NP_LOADDS NPP_Shutdown(void);
+NPError NP_LOADDS NPP_Initialize();
+void    NP_LOADDS NPP_Shutdown();
 NPError NP_LOADDS NPP_New(NPMIMEType pluginType, NPP instance,
                           uint16_t mode, int16_t argc, char* argn[],
                           char* argv[], NPSavedData* saved);
@@ -530,10 +553,14 @@ NPError     NP_LOADDS NPN_GetAuthenticationInfo(NPP instance,
                                                 char **username, uint32_t *ulen,
                                                 char **password,
                                                 uint32_t *plen);
+uint32_t    NP_LOADDS NPN_ScheduleTimer(NPP instance, uint32_t interval, NPBool repeat, void (*timerFunc)(NPP npp, uint32_t timerID));
+void        NP_LOADDS NPN_UnscheduleTimer(NPP instance, uint32_t timerID);
+NPError     NP_LOADDS NPN_PopUpContextMenu(NPP instance, NPMenu* menu);
+NPBool      NP_LOADDS NPN_ConvertPoint(NPP instance, double sourceX, double sourceY, NPCoordinateSpace sourceSpace, double *destX, double *destY, NPCoordinateSpace destSpace);
 
 
 #ifdef __cplusplus
 }  /* end extern "C" */
 #endif
 
-#endif /* _NPAPI_H_ */
+#endif /* npapi_h_ */
